@@ -81,22 +81,24 @@ class StreamHub(BaseHub, ABC):
             # removed by key
             return True
         # remove by value
-        for key in self.__channels:
-            if self.__channels.get(key) == channel:
-                self.__channels.pop(key, None)
+        keys = set(self.__channels.keys())
+        for address in keys:
+            if self.__channels.get(address) == channel:
+                self.__channels.pop(address, None)
                 return True
 
 
 class ServerHub(StreamHub, Runnable):
     """ Stream Server Hub """
 
-    def __init__(self, delegate: ConnectionDelegate):
+    def __init__(self, delegate: ConnectionDelegate, daemon: bool = False):
         super().__init__(delegate=delegate)
         self.__local_address: Optional[tuple] = None
         self.__master: Optional[socket.socket] = None
         # running thread
         self.__thread: Optional[Thread] = None
         self.__running = False
+        self.__daemon = daemon
 
     @property
     def running(self) -> bool:
@@ -127,7 +129,7 @@ class ServerHub(StreamHub, Runnable):
     def start(self):
         self.__force_stop()
         self.__running = True
-        t = Thread(target=self.run)
+        t = Thread(target=self.run, daemon=self.__daemon)
         self.__thread = t
         t.start()
 
@@ -152,7 +154,9 @@ class ServerHub(StreamHub, Runnable):
                     channel = StreamChannel(sock=sock, remote=address, local=self.__local_address)
                     self.put_channel(channel=channel)
             except socket.error as error:
-                print('[TCP] accepting connection error: %s' % error)
+                print('[TCP] socket error: %s' % error)
+            except Exception as error:
+                print('[TCP] accept error: %s' % error)
 
 
 class ClientHub(StreamHub):
@@ -182,7 +186,7 @@ def create_channel(remote: tuple, local: Optional[tuple]) -> Optional[Channel]:
             local = sock.getsockname()
         return StreamChannel(sock=sock, remote=remote, local=local)
     except socket.error as error:
-        print('[TCP] creating connection error: %s' % error)
+        print('[TCP] creating connection %s -> %s error: %s' % (local, remote, error))
 
 
 def create_socket(remote: tuple, local: Optional[tuple]) -> Optional[socket.socket]:
